package com.vcarecity.publish.controller;

import com.vcarecity.publish.annotation.SkipMethod;
import com.vcarecity.publish.api.ApiResult;
import com.vcarecity.publish.api.ApiStatus;
import com.vcarecity.publish.constants.ElasticConstant;
import com.vcarecity.publish.elastic.service.ElasticLoadDataService;
import com.vcarecity.publish.pojo.dto.UnitAgencyDTO;
import com.vcarecity.publish.pojo.dto.UnitIdDTO;
import com.vcarecity.publish.pojo.query.UnitAgencyQuery;
import com.vcarecity.publish.sql.service.UnitService;
import org.springframework.web.bind.annotation.*;

import javax.validation.Valid;
import java.io.IOException;
import java.util.List;

/**
 * @author Kerry on 19/12/11
 */

@RestController
public class MainController implements PrepareController {

    private int status = -1;

    private final UnitService unitService;
    private final ElasticLoadDataService elasticLoadDataService;

    public MainController(UnitService unitService, ElasticLoadDataService elasticLoadDataService) {
        this.unitService = unitService;
        this.elasticLoadDataService = elasticLoadDataService;
    }


    @RequestMapping(value = "/load", method = {RequestMethod.GET, RequestMethod.POST})
    public ApiResult loadData() {
        if (status == -1) {
            status = 0;
            synchronized (this) {
                new Thread(() -> {
                    try {
                        elasticLoadDataService.loadData();
                        status = 1;
                    } catch (IOException e) {
                        e.printStackTrace();
                        status = -1;
                    }
                }).start();
            }
        }
        return ApiResult.builder().status(40000).message("loading data").build();
    }


    @PostMapping("/query-unit-agency")
    public ApiResult queryUnitAgency(@RequestBody @Valid UnitAgencyQuery unitAgencyQuery) throws IOException {

        List<UnitIdDTO> data = unitService.getUnitAgencyByQuery(unitAgencyQuery, UnitIdDTO.class, ElasticConstant.UNIT_AGENCY_INDEX);

        return ApiResult.builder().status(ApiStatus.SUCCESS_CODE).data(data).build();
    }

    @PostMapping("/test/query-unit-agency")
    public ApiResult testQueryUnitAgency(@RequestBody @Valid UnitAgencyQuery unitAgencyQuery) throws IOException {
        List<UnitAgencyDTO> data = unitService.getUnitAgencyByQuery(unitAgencyQuery, UnitAgencyDTO.class, ElasticConstant.UNIT_AGENCY_INDEX);
        return ApiResult.builder().status(ApiStatus.SUCCESS_CODE).data(data).build();
    }

    @SkipMethod
    @Override
    public boolean isPrepare() {
        return false;
    }
}
